@@ -1,5 +1,6 @@
-#!/bin/bash
+#!/bin/sh -eu
 cargo build
+cargo xtask mangen
 
 ./target/debug/oxipng -V > MANUAL.txt
 #Redirect all streams to prevent detection of the terminal width and force an internal default of 100
