@@ -4,42 +4,42 @@
 #[cfg(not(feature = "parallel"))]
 use std::cell::RefCell;
 use std::sync::{
-    atomic::{AtomicUsize, Ordering::*},
     Arc,
+    atomic::{AtomicUsize, Ordering::*},
 };
 
 #[cfg(feature = "parallel")]
-use crossbeam_channel::{unbounded, Receiver, Sender};
-use deflate::Deflaters;
+use crossbeam_channel::{Receiver, Sender, unbounded};
+use deflate::Deflater;
 use indexmap::IndexSet;
 use log::trace;
 use rayon::prelude::*;
 
 #[cfg(not(feature = "parallel"))]
 use crate::rayon;
-use crate::{atomicmin::AtomicMin, deflate, filters::RowFilter, png::PngImage, Deadline, PngError};
+use crate::{
+    Deadline, PngError, atomicmin::AtomicMin, deflate, filters::FilterStrategy, png::PngImage,
+};
 
 pub(crate) struct Candidate {
     pub image: Arc<PngImage>,
-    pub idat_data: Vec<u8>,
-    pub filtered: Vec<u8>,
-    pub filter: RowFilter,
-    // For determining tie-breaker
+    pub idat_data: Option<Vec<u8>>,
+    pub estimated_output_size: usize,
+    /// The input filter, which is retained for printing and for APNG frames.
+    pub filter: FilterStrategy,
+    /// The filter returned by the filter function, which may be Predefined.
+    /// Use this for the next round to avoid recomputing the filter.
+    pub filter_used: FilterStrategy,
+    /// For determining tie-breaker
     nth: usize,
 }
 
 impl Candidate {
-    /// Return an estimate of the output size which can help with evaluation of very small data
-    #[must_use]
-    pub fn estimated_output_size(&self) -> usize {
-        self.idat_data.len() + self.image.key_chunks_size()
-    }
-
-    fn cmp_key(&self) -> impl Ord {
+    fn cmp_key(&self) -> impl Ord + use<> {
         (
-            self.estimated_output_size(),
+            self.estimated_output_size,
             self.image.data.len(),
-            self.filter,
+            self.filter.clone(),
             // Prefer the later image added (e.g. baseline, which is always added last)
             usize::MAX - self.nth,
         )
@@ -49,9 +49,10 @@ impl Candidate {
 /// Collect image versions and pick one that compresses best
 pub(crate) struct Evaluator {
     deadline: Arc<Deadline>,
-    filters: IndexSet<RowFilter>,
-    deflater: Deflaters,
+    filters: IndexSet<FilterStrategy>,
+    deflater: Deflater,
     optimize_alpha: bool,
+    final_round: bool,
     nth: AtomicUsize,
     executed: Arc<AtomicUsize>,
     best_candidate_size: Arc<AtomicMin>,
@@ -66,9 +67,10 @@ pub(crate) struct Evaluator {
 impl Evaluator {
     pub fn new(
         deadline: Arc<Deadline>,
-        filters: IndexSet<RowFilter>,
-        deflater: Deflaters,
+        filters: IndexSet<FilterStrategy>,
+        deflater: Deflater,
         optimize_alpha: bool,
+        final_round: bool,
     ) -> Self {
         #[cfg(feature = "parallel")]
         let eval_channel = unbounded();
@@ -77,6 +79,7 @@ impl Evaluator {
             filters,
             deflater,
             optimize_alpha,
+            final_round,
             nth: AtomicUsize::new(0),
             executed: Arc::new(AtomicUsize::new(0)),
             best_candidate_size: Arc::new(AtomicMin::new(None)),
@@ -127,6 +130,7 @@ impl Evaluator {
         let filters = self.filters.clone();
         let deflater = self.deflater;
         let optimize_alpha = self.optimize_alpha;
+        let final_round = self.final_round;
         let executed = self.executed.clone();
         let best_candidate_size = self.best_candidate_size.clone();
         let description = description.to_string();
@@ -142,28 +146,27 @@ impl Evaluator {
             // which are dangerous to do in side Rayon's loop.
             // Instead, only update (atomic) best size in real time,
             // and the best result later without need for locks.
-            filters_iter.for_each(|&filter| {
+            filters_iter.for_each(|filter| {
                 if deadline.passed() {
                     return;
                 }
-                let filtered = image.filter_image(filter, optimize_alpha);
+                let (filtered, filter_used) = image.filter_image(filter.clone(), optimize_alpha);
                 let idat_data = deflater.deflate(&filtered, best_candidate_size.get());
                 if let Ok(idat_data) = idat_data {
+                    let estimated_output_size = image.estimated_output_size(&idat_data);
+                    // We only need to retain the IDAT data in the final round
                     let new = Candidate {
                         image: image.clone(),
-                        idat_data,
-                        filtered,
-                        filter,
+                        idat_data: if final_round { Some(idat_data) } else { None },
+                        estimated_output_size,
+                        filter: filter.clone(),
+                        filter_used,
                         nth,
                     };
-                    let size = new.estimated_output_size();
-                    best_candidate_size.set_min(size);
+                    best_candidate_size.set_min(estimated_output_size);
                     trace!(
                         "Eval: {}-bit {:23} {:8}   {} bytes",
-                        image.ihdr.bit_depth,
-                        description,
-                        filter,
-                        size
+                        image.ihdr.bit_depth, description, filter, estimated_output_size
                     );
 
                     #[cfg(feature = "parallel")]
@@ -181,10 +184,7 @@ impl Evaluator {
                 } else if let Err(PngError::DeflatedDataTooLong(size)) = idat_data {
                     trace!(
                         "Eval: {}-bit {:23} {:8}  >{} bytes",
-                        image.ihdr.bit_depth,
-                        description,
-                        filter,
-                        size
+                        image.ihdr.bit_depth, description, filter, size
                     );
                 }
             });
