@@ -1,30 +1,67 @@
 use std::{fmt, fmt::Display, mem::transmute};
 
-use crate::error::PngError;
-
 /// Filtering strategy for use in [`Options`][crate::Options]
+#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Hash)]
+pub enum FilterStrategy {
+    /// Same filter for all rows
+    Basic(RowFilter),
+    /// Minimum sum of absolute differences
+    MinSum,
+    /// Shannon entropy
+    Entropy,
+    /// Count of distinct bigrams
+    Bigrams,
+    /// Shannon entropy of bigrams
+    BigEnt,
+    /// Deflate compression
+    Brute {
+        /// The number of lines to compress at once
+        num_lines: usize,
+        /// The compression level to use (1-12)
+        level: u8,
+    },
+    /// Predefined filter for each row
+    Predefined(Vec<RowFilter>),
+}
+
+impl FilterStrategy {
+    pub const NONE: Self = Self::Basic(RowFilter::None);
+    pub const SUB: Self = Self::Basic(RowFilter::Sub);
+    pub const UP: Self = Self::Basic(RowFilter::Up);
+    pub const AVERAGE: Self = Self::Basic(RowFilter::Average);
+    pub const PAETH: Self = Self::Basic(RowFilter::Paeth);
+}
+
+impl Display for FilterStrategy {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Basic(filter) => filter.fmt(f),
+            Self::MinSum => "MinSum".fmt(f),
+            Self::Entropy => "Entropy".fmt(f),
+            Self::Bigrams => "Bigrams".fmt(f),
+            Self::BigEnt => "BigEnt".fmt(f),
+            Self::Brute { .. } => "Brute".fmt(f),
+            Self::Predefined(_) => "Predefined".fmt(f),
+        }
+    }
+}
+
+/// PNG delta filters
 #[repr(u8)]
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Copy, Hash)]
 pub enum RowFilter {
-    // Standard filter types
     None,
     Sub,
     Up,
     Average,
     Paeth,
-    // Heuristic strategies
-    MinSum,
-    Entropy,
-    Bigrams,
-    BigEnt,
-    Brute,
 }
 
 impl TryFrom<u8> for RowFilter {
     type Error = ();
 
     fn try_from(value: u8) -> Result<Self, Self::Error> {
-        if value > Self::LAST {
+        if value > 4 {
             return Err(());
         }
         unsafe { transmute(value as i8) }
@@ -40,11 +77,6 @@ impl Display for RowFilter {
                 Self::Up => "Up",
                 Self::Average => "Average",
                 Self::Paeth => "Paeth",
-                Self::MinSum => "MinSum",
-                Self::Entropy => "Entropy",
-                Self::Bigrams => "Bigrams",
-                Self::BigEnt => "BigEnt",
-                Self::Brute => "Brute",
             },
             f,
         )
@@ -52,9 +84,7 @@ impl Display for RowFilter {
 }
 
 impl RowFilter {
-    pub const LAST: u8 = Self::Brute as u8;
-    pub(crate) const STANDARD: [Self; 5] =
-        [Self::None, Self::Sub, Self::Up, Self::Average, Self::Paeth];
+    pub(crate) const ALL: [Self; 5] = [Self::None, Self::Sub, Self::Up, Self::Average, Self::Paeth];
     pub(crate) const SINGLE_LINE: [Self; 2] = [Self::None, Self::Sub];
 
     pub(crate) fn filter_line(
@@ -97,25 +127,20 @@ impl RowFilter {
             }
             Self::Average => {
                 for (i, byte) in data.iter().enumerate() {
-                    buf.push(match i.checked_sub(bpp) {
-                        Some(x) => byte.wrapping_sub(
-                            ((u16::from(data[x]) + u16::from(prev_line[i])) >> 1) as u8,
-                        ),
-                        None => byte.wrapping_sub(prev_line[i] >> 1),
-                    });
+                    buf.push(byte.wrapping_sub(i.checked_sub(bpp).map_or_else(
+                        || prev_line[i] >> 1,
+                        |x| ((u16::from(data[x]) + u16::from(prev_line[i])) >> 1) as u8,
+                    )));
                 }
             }
             Self::Paeth => {
                 for (i, byte) in data.iter().enumerate() {
-                    buf.push(match i.checked_sub(bpp) {
-                        Some(x) => {
-                            byte.wrapping_sub(paeth_predictor(data[x], prev_line[i], prev_line[x]))
-                        }
-                        None => byte.wrapping_sub(prev_line[i]),
-                    });
+                    buf.push(byte.wrapping_sub(i.checked_sub(bpp).map_or_else(
+                        || prev_line[i],
+                        |x| paeth_predictor(data[x], prev_line[i], prev_line[x]),
+                    )));
                 }
             }
-            _ => unreachable!(),
         }
     }
 
@@ -139,10 +164,28 @@ impl RowFilter {
                         .unwrap_or(i),
                     _ => i - 1,
                 };
+
+                // These assertions help eliminate a few bounds checks in the slice accesses below
+                assert!(prev < pixels.len());
+                assert!(i < prev_pixels.len());
+
                 match self {
+                    Self::None => unreachable!(),
                     Self::Sub => {
-                        for j in 0..color_bytes {
-                            pixels[i][j] = pixels[prev][j];
+                        // The code below is roughly equivalent to pixels[i][0..color_bytes].copy_from_slice(&pixels[prev][0..color_bytes]),
+                        // if such a thing was possible to do without violating Rust aliasing rules. See:
+                        // https://users.rust-lang.org/t/problem-borrowing-two-elements-of-vec-mutably/21446/2
+
+                        if prev < i {
+                            let (pixels_head, pixels_tail) = pixels.split_at_mut(prev + 1);
+                            pixels_tail[i - prev - 1][0..color_bytes]
+                                .copy_from_slice(&pixels_head[prev][0..color_bytes]);
+                        } else if prev > i {
+                            let (pixels_head, pixels_tail) = pixels.split_at_mut(i + 1);
+                            pixels_head[i][0..color_bytes]
+                                .copy_from_slice(&pixels_tail[prev - i - 1][0..color_bytes]);
+                        } else {
+                            // If prev == i, we'd be copying the pixels onto themselves, which is useless
                         }
                     }
                     Self::Up => {
@@ -171,7 +214,6 @@ impl RowFilter {
                             };
                         }
                     }
-                    _ => unreachable!(),
                 }
             }
         }
@@ -183,7 +225,7 @@ impl RowFilter {
         data: &[u8],
         prev_line: &[u8],
         buf: &mut Vec<u8>,
-    ) -> Result<(), PngError> {
+    ) {
         buf.clear();
         buf.reserve(data.len());
         assert!(data.len() >= bpp);
@@ -195,10 +237,7 @@ impl RowFilter {
             Self::Sub => {
                 for (i, &cur) in data.iter().enumerate() {
                     let prev_byte = i.checked_sub(bpp).and_then(|x| buf.get(x).copied());
-                    buf.push(match prev_byte {
-                        Some(b) => cur.wrapping_add(b),
-                        None => cur,
-                    });
+                    buf.push(prev_byte.map_or(cur, |b| cur.wrapping_add(b)));
                 }
             }
             Self::Up => {
@@ -211,10 +250,10 @@ impl RowFilter {
             Self::Average => {
                 for (i, (&cur, &last)) in data.iter().zip(prev_line).enumerate() {
                     let prev_byte = i.checked_sub(bpp).and_then(|x| buf.get(x).copied());
-                    buf.push(match prev_byte {
-                        Some(b) => cur.wrapping_add(((u16::from(b) + u16::from(last)) >> 1) as u8),
-                        None => cur.wrapping_add(last >> 1),
-                    });
+                    buf.push(cur.wrapping_add(prev_byte.map_or_else(
+                        || last >> 1,
+                        |b| ((u16::from(b) + u16::from(last)) >> 1) as u8,
+                    )));
                 }
             }
             Self::Paeth => {
@@ -232,9 +271,7 @@ impl RowFilter {
                     );
                 }
             }
-            _ => return Err(PngError::InvalidData),
         }
-        Ok(())
     }
 }
 
