@@ -1,14 +1,13 @@
 use indexmap::IndexSet;
-use log::warn;
+use log::{debug, trace, warn};
 use rgb::{RGB16, RGBA8};
 
 use crate::{
+    Deflater, Options, PngResult,
     colors::{BitDepth, ColorType},
     deflate::{crc32, inflate},
     display_chunks::DISPLAY_CHUNKS,
     error::PngError,
-    interlace::Interlacing,
-    Deflaters, PngResult,
 };
 
 #[derive(Debug, Clone)]
@@ -22,8 +21,8 @@ pub struct IhdrData {
     pub color_type: ColorType,
     /// The bit depth of the image
     pub bit_depth: BitDepth,
-    /// The interlacing mode of the image
-    pub interlaced: Interlacing,
+    /// Whether the image is interlaced
+    pub interlaced: bool,
 }
 
 impl IhdrData {
@@ -36,18 +35,16 @@ impl IhdrData {
 
     /// Byte length of IDAT that is correct for this IHDR
     #[must_use]
-    pub fn raw_data_size(&self) -> usize {
+    pub const fn raw_data_size(&self) -> usize {
         let w = self.width as usize;
         let h = self.height as usize;
         let bpp = self.bpp();
 
-        fn bitmap_size(bpp: usize, w: usize, h: usize) -> usize {
+        const fn bitmap_size(bpp: usize, w: usize, h: usize) -> usize {
             (w * bpp).div_ceil(8) * h
         }
 
-        if self.interlaced == Interlacing::None {
-            bitmap_size(bpp, w, h) + h
-        } else {
+        if self.interlaced {
             let mut size = bitmap_size(bpp, (w + 7) >> 3, (h + 7) >> 3) + ((h + 7) >> 3);
             if w > 4 {
                 size += bitmap_size(bpp, (w + 3) >> 3, (h + 7) >> 3) + ((h + 7) >> 3);
@@ -61,6 +58,8 @@ impl IhdrData {
                 size += bitmap_size(bpp, w >> 1, (h + 1) >> 1) + ((h + 1) >> 1);
             }
             size + bitmap_size(bpp, w, h >> 1) + (h >> 1)
+        } else {
+            bitmap_size(bpp, w, h) + h
         }
     }
 }
@@ -173,10 +172,7 @@ pub fn parse_next_chunk<'a>(
 
     let chunk_bytes = &byte_data[chunk_start..chunk_start + 4 + length as usize];
     if !fix_errors && crc32(chunk_bytes) != crc {
-        return Err(PngError::new(&format!(
-            "CRC Mismatch in {} chunk; May be recoverable by using --fix",
-            String::from_utf8_lossy(chunk_name)
-        )));
+        return Err(PngError::CRCMismatch(chunk_name.try_into().unwrap()));
     }
 
     let name: [u8; 4] = chunk_name.try_into().unwrap();
@@ -209,12 +205,16 @@ pub fn parse_ihdr_chunk(
             },
             4 => ColorType::GrayscaleAlpha,
             6 => ColorType::RGBA,
-            _ => return Err(PngError::new("Unexpected color type in header")),
+            _ => return Err(PngError::InvalidData),
         },
         bit_depth: byte_data[8].try_into()?,
         width: read_be_u32(&byte_data[0..4]),
         height: read_be_u32(&byte_data[4..8]),
-        interlaced: interlaced.try_into()?,
+        interlaced: match interlaced {
+            0 => false,
+            1 => true,
+            _ => return Err(PngError::InvalidData),
+        },
     })
 }
 
@@ -223,7 +223,7 @@ fn palette_to_rgba(
     palette_data: Option<Vec<u8>>,
     trns_data: Option<Vec<u8>>,
 ) -> Result<Vec<RGBA8>, PngError> {
-    let palette_data = palette_data.ok_or_else(|| PngError::new("no palette in indexed image"))?;
+    let palette_data = palette_data.ok_or(PngError::ChunkMissing("PLTE"))?;
     let mut palette: Vec<_> = palette_data
         .chunks_exact(3)
         .map(|color| RGBA8::new(color[0], color[1], color[2], 255))
@@ -248,7 +248,7 @@ pub fn read_be_u32(bytes: &[u8]) -> u32 {
 }
 
 /// Extract and decompress the ICC profile from an iCCP chunk
-pub fn extract_icc(iccp: &Chunk) -> Option<Vec<u8>> {
+pub fn extract_icc(iccp: &Chunk, max_size: Option<usize>) -> Option<Vec<u8>> {
     // Skip (useless) profile name
     let mut data = iccp.data.as_slice();
     loop {
@@ -264,19 +264,22 @@ pub fn extract_icc(iccp: &Chunk) -> Option<Vec<u8>> {
         return None; // The profile is supposed to be compressed (method 0)
     }
     // The decompressed size is unknown so we have to guess the required buffer size
-    let max_size = compressed_data.len() * 2 + 1000;
-    match inflate(compressed_data, max_size) {
+    let mut out_size = compressed_data.len() * 2 + 1000;
+    if let Some(max) = max_size {
+        out_size = out_size.min(max);
+    }
+    match inflate(compressed_data, out_size) {
         Ok(icc) => Some(icc),
         Err(e) => {
             // Log the error so we can know if the buffer size needs to be adjusted
-            warn!("Failed to decompress icc: {}", e);
+            warn!("Failed to decompress icc: {e}");
             None
         }
     }
 }
 
 /// Make an iCCP chunk by compressing the ICC profile
-pub fn make_iccp(icc: &[u8], deflater: Deflaters, max_size: Option<usize>) -> PngResult<Chunk> {
+pub fn make_iccp(icc: &[u8], deflater: Deflater, max_size: Option<usize>) -> PngResult<Chunk> {
     let mut compressed = deflater.deflate(icc, max_size)?;
     let mut data = Vec::with_capacity(compressed.len() + 5);
     data.extend(b"icc"); // Profile name - generally unused, can be anything
@@ -314,3 +317,109 @@ pub fn srgb_rendering_intent(icc_data: &[u8]) -> Option<u8> {
         _ => None,
     }
 }
+
+/// Process aux chunks and potentially adjust options before optimizing
+pub fn preprocess_chunks(aux_chunks: &mut Vec<Chunk>, opts: &mut Options) {
+    let has_srgb = aux_chunks.iter().any(|c| &c.name == b"sRGB");
+    // Grayscale conversion should not be performed if the image is not in the sRGB colorspace
+    // An sRGB profile would need to be stripped on conversion, so disallow if stripping is disabled
+    let mut allow_grayscale = !has_srgb || opts.strip != StripChunks::None;
+
+    if let Some(iccp_idx) = aux_chunks.iter().position(|c| &c.name == b"iCCP") {
+        allow_grayscale = false;
+        // See if we can replace an iCCP chunk with an sRGB chunk
+        let may_replace_iccp = opts.strip != StripChunks::None && opts.strip.keep(b"sRGB");
+        if may_replace_iccp && has_srgb {
+            // Files aren't supposed to have both chunks, so we chose to honor sRGB
+            trace!("Removing iCCP chunk due to conflict with sRGB chunk");
+            aux_chunks.remove(iccp_idx);
+            allow_grayscale = true;
+        } else if let Some(icc) = extract_icc(&aux_chunks[iccp_idx], opts.max_decompressed_size) {
+            let intent = if may_replace_iccp {
+                srgb_rendering_intent(&icc)
+            } else {
+                None
+            };
+            // sRGB-like profile can be replaced with an sRGB chunk with the same rendering intent
+            if let Some(intent) = intent {
+                trace!("Replacing iCCP chunk with equivalent sRGB chunk");
+                aux_chunks[iccp_idx] = Chunk {
+                    name: *b"sRGB",
+                    data: vec![intent],
+                };
+                allow_grayscale = true;
+            } else if opts.idat_recoding {
+                // Try recompressing the profile
+                let cur_len = aux_chunks[iccp_idx].data.len();
+                if let Ok(iccp) = make_iccp(&icc, opts.deflater, Some(cur_len - 1)) {
+                    debug!(
+                        "Recompressed iCCP chunk: {} ({} bytes decrease)",
+                        iccp.data.len(),
+                        cur_len - iccp.data.len()
+                    );
+                    aux_chunks[iccp_idx] = iccp;
+                }
+            }
+        }
+    }
+
+    if !allow_grayscale && opts.grayscale_reduction {
+        debug!("Disabling grayscale reduction due to presence of sRGB or iCCP chunk");
+        opts.grayscale_reduction = false;
+    }
+
+    // Check for APNG by presence of acTL chunk
+    if aux_chunks.iter().any(|c| &c.name == b"acTL") {
+        warn!("APNG detected, disabling all reductions");
+        opts.interlace = None;
+        opts.bit_depth_reduction = false;
+        opts.color_type_reduction = false;
+        opts.palette_reduction = false;
+        opts.grayscale_reduction = false;
+    }
+}
+
+/// Perform cleanup of certain aux chunks after optimization has been completed
+pub fn postprocess_chunks(aux_chunks: &mut Vec<Chunk>, ihdr: &IhdrData, orig_ihdr: &IhdrData) {
+    // If the depth/color type has changed, some chunks may be invalid and should be dropped
+    // While these could potentially be converted, they have no known use case today and are
+    // generally more trouble than they're worth
+    if orig_ihdr.bit_depth != ihdr.bit_depth || orig_ihdr.color_type != ihdr.color_type {
+        aux_chunks.retain(|c| {
+            let invalid = &c.name == b"bKGD" || &c.name == b"sBIT" || &c.name == b"hIST";
+            if invalid {
+                warn!(
+                    "Removing {} chunk as it no longer matches the image data",
+                    std::str::from_utf8(&c.name).unwrap()
+                );
+            }
+            !invalid
+        });
+    }
+
+    // Remove any sRGB or iCCP chunks if the image was converted to or from grayscale
+    if orig_ihdr.color_type.is_gray() != ihdr.color_type.is_gray() {
+        aux_chunks.retain(|c| {
+            let invalid = &c.name == b"sRGB" || &c.name == b"iCCP";
+            if invalid {
+                trace!(
+                    "Removing {} chunk as it no longer matches the color type",
+                    std::str::from_utf8(&c.name).unwrap()
+                );
+            }
+            !invalid
+        });
+    }
+
+    // Remove iDOT which will necessarily be invalid after successful optimization
+    aux_chunks.retain(|c| {
+        let invalid = &c.name == b"iDOT";
+        if invalid {
+            trace!(
+                "Removing {} chunk as it no longer matches the IDAT",
+                std::str::from_utf8(&c.name).unwrap()
+            );
+        }
+        !invalid
+    });
+}
