@@ -1,23 +1,4 @@
-#![warn(trivial_casts, trivial_numeric_casts, unused_import_braces)]
-#![deny(missing_debug_implementations, missing_copy_implementations)]
-#![warn(clippy::expl_impl_clone_on_copy)]
-#![warn(clippy::float_cmp_const)]
-#![warn(clippy::linkedlist)]
-#![warn(clippy::map_flatten)]
-#![warn(clippy::match_same_arms)]
-#![warn(clippy::mem_forget)]
-#![warn(clippy::mut_mut)]
-#![warn(clippy::mutex_integer)]
-#![warn(clippy::needless_continue)]
-#![warn(clippy::path_buf_push_overwrite)]
-#![warn(clippy::range_plus_one)]
-#![allow(clippy::cognitive_complexity)]
-#![allow(clippy::upper_case_acronyms)]
-#![cfg_attr(
-    not(feature = "zopfli"),
-    allow(irrefutable_let_patterns),
-    allow(unreachable_patterns)
-)]
+#![cfg_attr(not(feature = "zopfli"), allow(unreachable_patterns))]
 
 #[cfg(feature = "parallel")]
 extern crate rayon;
@@ -27,27 +8,28 @@ mod rayon;
 
 use std::{
     fs::{File, Metadata},
-    io::{stdin, stdout, BufWriter, Read, Write},
-    path::Path,
+    io::{BufWriter, Read, Write, stdin, stdout},
+    path::{Path, PathBuf},
     sync::{
-        atomic::{AtomicBool, Ordering},
         Arc,
+        atomic::{AtomicBool, Ordering},
     },
     time::{Duration, Instant},
 };
 
-pub use indexmap::{indexset, IndexSet};
+pub use indexmap::{IndexSet, indexset};
 use log::{debug, info, trace, warn};
 use rayon::prelude::*;
 pub use rgb::{RGB16, RGBA8};
 
+#[cfg(feature = "zopfli")]
+pub use crate::deflate::ZopfliOptions;
 pub use crate::{
     colors::{BitDepth, ColorType},
-    deflate::Deflaters,
+    deflate::Deflater,
     error::PngError,
-    filters::RowFilter,
+    filters::{FilterStrategy, RowFilter},
     headers::StripChunks,
-    interlace::Interlacing,
     options::{InFile, Options, OutFile},
 };
 use crate::{
@@ -130,7 +112,7 @@ impl RawImage {
                     height,
                     color_type,
                     bit_depth,
-                    interlaced: Interlacing::None,
+                    interlaced: false,
                 },
                 data,
             }),
@@ -146,7 +128,7 @@ impl RawImage {
     /// Add an ICC profile for the image
     pub fn add_icc_profile(&mut self, data: &[u8]) {
         // Compress with fastest compression level - will be recompressed during optimization
-        let deflater = Deflaters::Libdeflater { compression: 1 };
+        let deflater = Deflater::Libdeflater { compression: 1 };
         if let Ok(iccp) = make_iccp(data, deflater, None) {
             self.aux_chunks.push(iccp);
         }
@@ -170,7 +152,7 @@ impl RawImage {
 
         let mut png = PngData {
             raw: result.image,
-            idat_data: result.data,
+            idat_data: result.idat_data.unwrap(),
             aux_chunks,
             frames: Vec::new(),
         };
@@ -181,7 +163,9 @@ impl RawImage {
 }
 
 /// Perform optimization on the input file using the options provided
-pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<()> {
+///
+/// Returns the original and optimized file sizes
+pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(usize, usize)> {
     // Read in the file and try to decode as PNG.
     info!("Processing: {input}");
 
@@ -218,7 +202,7 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
             let mut data = Vec::new();
             stdin()
                 .read_to_end(&mut data)
-                .map_err(|e| PngError::new(&format!("Error reading stdin: {e}")))?;
+                .map_err(|e| PngError::ReadFailed("stdin".into(), e))?;
             data
         }
     };
@@ -230,14 +214,14 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
 
     let in_length = in_data.len();
 
-    if is_fully_optimized(in_data.len(), optimized_output.len(), opts) {
+    if is_fully_optimized(in_length, optimized_output.len(), opts) {
         match (output, input) {
-            // if p is None, it also means same as the input path
-            (OutFile::Path { path, .. }, InFile::Path(ref input_path))
-                if path.as_ref().map_or(true, |p| p == input_path) =>
+            // If output path is None, it also means same as the input path
+            (OutFile::Path { path, .. }, InFile::Path(input_path))
+                if path.as_ref().is_none_or(|p| p == input_path) =>
             {
                 info!("{input}: Could not optimize further, no change written");
-                return Ok(());
+                return Ok((in_length, in_length));
             }
             _ => {
                 optimized_output = in_data;
@@ -261,26 +245,20 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
 
     match (output, input) {
         (OutFile::None, _) => {
-            info!("{savings}: Running in pretend mode, no output");
+            info!("{savings}: Dry run, no output");
         }
         (&OutFile::StdOut, _) | (&OutFile::Path { path: None, .. }, &InFile::StdIn) => {
             let mut buffer = BufWriter::new(stdout());
             buffer
                 .write_all(&optimized_output)
-                .map_err(|e| PngError::new(&format!("Unable to write to stdout: {e}")))?;
+                .map_err(|e| PngError::WriteFailed("stdout".into(), e))?;
         }
         (OutFile::Path { path, .. }, _) => {
             let output_path = path
                 .as_ref()
-                .map(|p| p.as_path())
-                .unwrap_or_else(|| input.path().unwrap());
-            let out_file = File::create(output_path).map_err(|err| {
-                PngError::new(&format!(
-                    "Unable to write to file {}: {}",
-                    output_path.display(),
-                    err
-                ))
-            })?;
+                .map_or_else(|| input.path().unwrap(), PathBuf::as_path);
+            let out_file = File::create(output_path)
+                .map_err(|err| PngError::WriteFailed(output_path.display().to_string(), err))?;
             if let Some(metadata_input) = &opt_metadata_preserved {
                 copy_permissions(metadata_input, &out_file)?;
             }
@@ -290,13 +268,7 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
                 .write_all(&optimized_output)
                 // flush BufWriter so IO errors don't get swallowed silently on close() by drop!
                 .and_then(|()| buffer.flush())
-                .map_err(|e| {
-                    PngError::new(&format!(
-                        "Unable to write to {}: {}",
-                        output_path.display(),
-                        e
-                    ))
-                })?;
+                .map_err(|e| PngError::WriteFailed(output_path.display().to_string(), e))?;
             // force drop and thereby closing of file handle before modifying any timestamp
             std::mem::drop(buffer);
             if let Some(metadata_input) = &opt_metadata_preserved {
@@ -305,7 +277,7 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
             info!("{}: {}", savings, output_path.display());
         }
     }
-    Ok(())
+    Ok((in_length, optimized_output.len()))
 }
 
 /// Perform optimization on the input file using the options provided, where the file is already
@@ -359,7 +331,7 @@ fn optimize_png(
     };
     if let Some(result) = optimize_raw(raw.clone(), &opts, deadline.clone(), max_size) {
         png.raw = result.image;
-        png.idat_data = result.data;
+        png.idat_data = result.idat_data.unwrap();
         recompress_frames(png, &opts, deadline, result.filter)?;
         postprocess_chunks(&mut png.aux_chunks, &png.raw.ihdr, &raw.ihdr);
     }
@@ -395,8 +367,10 @@ fn optimize_png(
         );
     }
 
-    if opts.interlace == Some(Interlacing::Adam7) && png.raw.ihdr.interlaced != Interlacing::Adam7 {
-        warn!("Interlacing was not enabled as it would result in a larger file. To override this, use `--force`.");
+    if opts.interlace == Some(true) && !png.raw.ihdr.interlaced {
+        warn!(
+            "Interlacing was not enabled as it would result in a larger file. To override this, use `--force`."
+        );
     }
 
     #[cfg(feature = "sanity-checks")]
@@ -418,19 +392,19 @@ fn optimize_raw(
     // 8 is a little slower but not noticeably when used only for reductions (o3 and higher)
     // 9 is not appreciably better than 8
     // 10 and higher are quite slow - good for filters but only good for reductions if matching the main zc level
-    let compression = match opts.deflate {
-        Deflaters::Libdeflater { compression } => {
+    let compression = match opts.deflater {
+        Deflater::Libdeflater { compression } => {
             if opts.fast_evaluation { 7 } else { 8 }.min(compression)
         }
         _ => 8,
     };
-    let eval_deflater = Deflaters::Libdeflater { compression };
+    let eval_deflater = Deflater::Libdeflater { compression };
     // If only one filter is selected, use this for evaluations
-    let eval_filters = if opts.filter.len() == 1 {
-        opts.filter.clone()
+    let eval_filters = if opts.filters.len() == 1 {
+        opts.filters.clone()
     } else {
         // None and Bigrams work well together, especially for alpha reductions
-        indexset! {RowFilter::None, RowFilter::Bigrams}
+        indexset! {FilterStrategy::NONE, FilterStrategy::Bigrams}
     };
     // This will collect all versions of images and pick one that compresses best
     let eval = Evaluator::new(
@@ -438,7 +412,7 @@ fn optimize_raw(
         eval_filters.clone(),
         eval_deflater,
         false,
-        opts.deflate == eval_deflater,
+        opts.deflater == eval_deflater,
     );
     let mut new_image = perform_reductions(image.clone(), opts, &deadline, &eval);
     let eval_result = eval.get_best_candidate();
@@ -455,23 +429,23 @@ fn optimize_raw(
 
     let (result, deflater) = if opts.idat_recoding || reduction_occurred {
         let result = perform_trials(
-            new_image.clone(),
+            new_image,
             opts,
-            deadline.clone(),
+            deadline,
             max_size,
             eval_result,
             eval_filters,
             eval_deflater,
         );
-        (result?, opts.deflate)
+        (result?, opts.deflater)
     } else {
         // If idat_recoding is off and reductions were attempted but ended up choosing the baseline,
         // we should still check if the evaluator compressed the baseline smaller than the original.
         (eval_result?, eval_deflater)
     };
 
-    if result.data_is_compressed
-        && max_size.map_or(true, |max_size| result.estimated_output_size < max_size)
+    if result.idat_data.is_some()
+        && max_size.is_none_or(|max_size| result.estimated_output_size < max_size)
     {
         debug!("Found better result:");
         debug!("    {}, f = {}", deflater, result.filter);
@@ -487,27 +461,27 @@ fn perform_trials(
     deadline: Arc<Deadline>,
     max_size: Option<usize>,
     mut eval_result: Option<Candidate>,
-    eval_filters: IndexSet<RowFilter>,
-    eval_deflater: Deflaters,
+    eval_filters: IndexSet<FilterStrategy>,
+    eval_deflater: Deflater,
 ) -> Option<Candidate> {
-    let mut filters = opts.filter.clone();
+    let mut filters = opts.filters.clone();
     let fast_eval = opts.fast_evaluation && (filters.len() > 1 || eval_result.is_some());
     if fast_eval {
         // Perform a fast evaluation of selected filters followed by a single main compression trial
 
         if eval_result.is_some() {
             // Some filters have already been evaluated, we don't need to try them again
-            filters = filters.difference(&eval_filters).copied().collect();
+            filters = filters.difference(&eval_filters).cloned().collect();
         }
 
         if !filters.is_empty() {
             trace!("Evaluating {} filters", filters.len());
             let eval = Evaluator::new(
-                deadline.clone(),
+                deadline,
                 filters,
                 eval_deflater,
                 opts.optimize_alpha,
-                opts.deflate == eval_deflater,
+                opts.deflater == eval_deflater,
             );
             if let Some(result) = &eval_result {
                 eval.set_best_size(result.estimated_output_size);
@@ -521,21 +495,21 @@ fn perform_trials(
         // We should have a result here - fail if not (e.g. deadline passed)
         let mut result = eval_result?;
 
-        if !result.data_is_compressed {
+        if result.idat_data.is_none() {
             // Compress with the main deflater
-            debug!("Trying filter {} with {}", result.filter, opts.deflate);
-            match opts.deflate.deflate(&result.data, max_size) {
+            debug!("Trying filter {} with {}", result.filter, opts.deflater);
+            let (data, _) = image.filter_image(result.filter_used.clone(), opts.optimize_alpha);
+            match opts.deflater.deflate(&data, max_size) {
                 Ok(idat_data) => {
                     result.estimated_output_size = result.image.estimated_output_size(&idat_data);
-                    result.data = idat_data;
-                    result.data_is_compressed = true;
+                    result.idat_data = Some(idat_data);
                     trace!("{} bytes", result.estimated_output_size);
                 }
                 Err(PngError::DeflatedDataTooLong(bytes)) => {
                     trace!(">{bytes} bytes");
                 }
                 Err(_) => (),
-            };
+            }
         }
         return Some(result);
     }
@@ -546,15 +520,15 @@ fn perform_trials(
         // Pick a filter automatically
         if image.ihdr.bit_depth as u8 >= 8 {
             // Bigrams is the best all-rounder when there's at least one byte per pixel
-            filters.insert(RowFilter::Bigrams);
+            filters.insert(FilterStrategy::Bigrams);
         } else {
             // Otherwise delta filters generally don't work well, so just stick with None
-            filters.insert(RowFilter::None);
+            filters.insert(FilterStrategy::NONE);
         }
     }
 
-    debug!("Trying {} filters with {}", filters.len(), opts.deflate);
-    let eval = Evaluator::new(deadline, filters, opts.deflate, opts.optimize_alpha, true);
+    debug!("Trying {} filters with {}", filters.len(), opts.deflater);
+    let eval = Evaluator::new(deadline, filters, opts.deflater, opts.optimize_alpha, true);
     if let Some(max_size) = max_size {
         eval.set_best_size(max_size);
     }
@@ -614,9 +588,14 @@ impl Deadline {
 
 /// Display the format of the image data
 fn report_format(prefix: &str, png: &PngImage) {
+    let interlaced = if png.ihdr.interlaced {
+        "interlaced"
+    } else {
+        "non-interlaced"
+    };
     debug!(
         "{}{}-bit {}, {}",
-        prefix, png.ihdr.bit_depth, png.ihdr.color_type, png.ihdr.interlaced
+        prefix, png.ihdr.bit_depth, png.ihdr.color_type, interlaced
     );
 }
 
@@ -625,11 +604,13 @@ fn recompress_frames(
     png: &mut PngData,
     opts: &Options,
     deadline: Arc<Deadline>,
-    filter: RowFilter,
+    filter: FilterStrategy,
 ) -> PngResult<()> {
     if !opts.idat_recoding || png.frames.is_empty() {
         return Ok(());
     }
+    // Ensure we don't try to recompress frames with a predefined filter
+    debug_assert!(!matches!(filter, FilterStrategy::Predefined { .. }));
     png.frames
         .par_iter_mut()
         .with_max_len(1)
@@ -642,9 +623,9 @@ fn recompress_frames(
             ihdr.width = frame.width;
             ihdr.height = frame.height;
             let image = PngImage::new(ihdr, &frame.data)?;
-            let filtered = image.filter_image(filter, opts.optimize_alpha);
+            let (filtered, _) = image.filter_image(filter.clone(), opts.optimize_alpha);
             let max_size = Some(frame.data.len() - 1);
-            if let Ok(data) = opts.deflate.deflate(&filtered, max_size) {
+            if let Ok(data) = opts.deflater.deflate(&filtered, max_size) {
                 debug!(
                     "Recompressed fdAT #{:<2}: {} ({} bytes decrease)",
                     i,
@@ -658,7 +639,7 @@ fn recompress_frames(
 }
 
 /// Check if an image was already optimized prior to oxipng's operations
-fn is_fully_optimized(original_size: usize, optimized_size: usize, opts: &Options) -> bool {
+const fn is_fully_optimized(original_size: usize, optimized_size: usize, opts: &Options) -> bool {
     original_size <= optimized_size && !opts.force
 }
 
@@ -667,24 +648,23 @@ fn copy_permissions(metadata_input: &Metadata, out_file: &File) -> PngResult<()>
         .set_permissions(metadata_input.permissions())
         .map_err(|err_io| {
             PngError::new(&format!(
-                "unable to set permissions for output file: {err_io}"
+                "Unable to set permissions for output file: {err_io}"
             ))
         })
 }
 
 #[cfg(not(feature = "filetime"))]
-fn copy_times(_: &Metadata, _: &Path) -> PngResult<()> {
+const fn copy_times(_: &Metadata, _: &Path) -> PngResult<()> {
     Ok(())
 }
 
 #[cfg(feature = "filetime")]
 fn copy_times(input_path_meta: &Metadata, out_path: &Path) -> PngResult<()> {
-    let atime = filetime::FileTime::from_last_access_time(input_path_meta);
     let mtime = filetime::FileTime::from_last_modification_time(input_path_meta);
-    trace!("attempting to set file times: atime: {atime:?}, mtime: {mtime:?}");
-    filetime::set_file_times(out_path, atime, mtime).map_err(|err_io| {
+    trace!("attempting to set file modification time: {mtime:?}");
+    filetime::set_file_mtime(out_path, mtime).map_err(|err_io| {
         PngError::new(&format!(
-            "unable to set file times on {out_path:?}: {err_io}"
+            "Unable to set file times on {out_path:?}: {err_io}"
         ))
     })
 }
