@@ -1,9 +1,4 @@
-use std::{
-    fs::File,
-    io::{BufReader, Read, Write},
-    path::Path,
-    sync::Arc,
-};
+use std::{fs, io::Write, path::Path, sync::Arc};
 
 use bitvec::bitarr;
 use libdeflater::{CompressionLvl, Compressor};
@@ -12,25 +7,20 @@ use rgb::ComponentSlice;
 use rustc_hash::FxHashMap;
 
 use crate::{
+    Options,
     apng::*,
     colors::{BitDepth, ColorType},
     deflate,
     error::PngError,
     filters::*,
     headers::*,
-    interlace::{deinterlace_image, interlace_image, Interlacing},
-    Options,
+    interlace::{deinterlace_image, interlace_image},
 };
 
 pub(crate) mod scan_lines;
 
 use self::scan_lines::ScanLines;
 
-/// Compression level to use for the Brute filter strategy
-const BRUTE_LEVEL: i32 = 1; // 1 is fastest, 2-4 are not useful, 5 is slower but more effective
-/// Number of lines to compress with the Brute filter strategy
-const BRUTE_LINES: usize = 4; // Values over 8 are generally not useful
-
 #[derive(Debug, Clone)]
 pub struct PngImage {
     /// The headers stored in the IHDR chunk
@@ -62,28 +52,7 @@ impl PngData {
     }
 
     pub fn read_file(filepath: &Path) -> Result<Vec<u8>, PngError> {
-        let file = match File::open(filepath) {
-            Ok(f) => f,
-            Err(_) => return Err(PngError::new("Failed to open file for reading")),
-        };
-        let file_len = file.metadata().map(|m| m.len() as usize).unwrap_or(0);
-        let mut reader = BufReader::new(file);
-        // Check file for PNG header
-        let mut header = [0; 8];
-        if reader.read_exact(&mut header).is_err() {
-            return Err(PngError::new("Not a PNG file: too small"));
-        }
-        if !file_header_is_valid(&header) {
-            return Err(PngError::new("Invalid PNG header detected"));
-        }
-        // Read raw png data into memory
-        let mut byte_data: Vec<u8> = Vec::with_capacity(file_len);
-        byte_data.extend_from_slice(&header);
-        match reader.read_to_end(&mut byte_data) {
-            Ok(_) => (),
-            Err(_) => return Err(PngError::new("Failed to read from file")),
-        }
-        Ok(byte_data)
+        fs::read(filepath).map_err(|e| PngError::ReadFailed(filepath.display().to_string(), e))
     }
 
     /// Create a new `PngData` struct by reading a slice
@@ -154,7 +123,7 @@ impl PngData {
                     });
                 }
                 b"acTL" => {
-                    warn!("Stripping animation data from APNG - image will become standard PNG")
+                    warn!("Stripping animation data from APNG - image will become standard PNG");
                 }
                 _ => (),
             }
@@ -164,15 +133,19 @@ impl PngData {
         if idat_data.is_empty() {
             return Err(PngError::ChunkMissing("IDAT"));
         }
-        let ihdr_chunk = match key_chunks.remove(b"IHDR") {
-            Some(ihdr) => ihdr,
-            None => return Err(PngError::ChunkMissing("IHDR")),
+        let Some(ihdr_chunk) = key_chunks.remove(b"IHDR") else {
+            return Err(PngError::ChunkMissing("IHDR"));
         };
         let ihdr = parse_ihdr_chunk(
             &ihdr_chunk,
             key_chunks.remove(b"PLTE"),
             key_chunks.remove(b"tRNS"),
         )?;
+        if let Some(max) = opts.max_decompressed_size {
+            if ihdr.raw_data_size() > max {
+                return Err(PngError::InflatedDataTooLong(max));
+            }
+        }
 
         let raw = PngImage::new(ihdr, &idat_data)?;
 
@@ -185,12 +158,6 @@ impl PngData {
         })
     }
 
-    /// Return an estimate of the output size which can help with evaluation of very small data
-    #[must_use]
-    pub fn estimated_output_size(&self) -> usize {
-        self.idat_data.len() + self.raw.key_chunks_size()
-    }
-
     /// Format the `PngData` struct into a valid PNG bytestream
     #[must_use]
     pub fn output(&self) -> Vec<u8> {
@@ -299,18 +266,17 @@ impl PngImage {
         Ok(image)
     }
 
-    /// Convert the image to the specified interlacing type
-    /// Returns true if the interlacing was changed, false otherwise
-    /// The `interlace` parameter specifies the *new* interlacing mode
+    /// Enable or disable interlacing
+    /// Returns the new image if the interlacing was changed, None otherwise
     /// Assumes that the data has already been de-filtered
     #[inline]
     #[must_use]
-    pub fn change_interlacing(&self, interlace: Interlacing) -> Option<Self> {
+    pub fn change_interlacing(&self, interlace: bool) -> Option<Self> {
         if interlace == self.ihdr.interlaced {
             return None;
         }
 
-        Some(if interlace == Interlacing::Adam7 {
+        Some(if interlace {
             // Convert progressive to interlaced data
             interlace_image(self)
         } else {
@@ -343,11 +309,10 @@ impl PngImage {
         match &self.ihdr.color_type {
             ColorType::Indexed { palette } => {
                 let plte = 12 + palette.len() * 3;
-                if let Some(trns) = palette.iter().rposition(|p| p.a != 255) {
-                    plte + 12 + trns + 1
-                } else {
-                    plte
-                }
+                palette
+                    .iter()
+                    .rposition(|p| p.a != 255)
+                    .map_or(plte, |trns| plte + 12 + trns + 1)
             }
             ColorType::Grayscale { transparent_shade } if transparent_shade.is_some() => 12 + 2,
             ColorType::RGB { transparent_color } if transparent_color.is_some() => 12 + 6,
@@ -355,6 +320,12 @@ impl PngImage {
         }
     }
 
+    /// Return an estimate of the output size which can help with evaluation of very small data
+    #[must_use]
+    pub fn estimated_output_size(&self, idat_data: &[u8]) -> usize {
+        idat_data.len() + self.key_chunks_size()
+    }
+
     /// Return an iterator over the scanlines of the image
     #[inline]
     #[must_use]
@@ -375,8 +346,8 @@ impl PngImage {
                 last_pass = line.pass;
             }
             last_line.resize(line.data.len(), 0);
-            let filter = RowFilter::try_from(line.filter).map_err(|_| PngError::InvalidData)?;
-            filter.unfilter_line(bpp, line.data, &last_line, &mut unfiltered_buf)?;
+            let filter = RowFilter::try_from(line.filter).map_err(|()| PngError::InvalidData)?;
+            filter.unfilter_line(bpp, line.data, &last_line, &mut unfiltered_buf);
             unfiltered.extend_from_slice(&unfiltered_buf);
             std::mem::swap(&mut last_line, &mut unfiltered_buf);
             unfiltered_buf.clear();
@@ -386,7 +357,11 @@ impl PngImage {
 
     /// Apply the specified filter type to all rows in the image
     #[must_use]
-    pub fn filter_image(&self, filter: RowFilter, optimize_alpha: bool) -> Vec<u8> {
+    pub fn filter_image(
+        &self,
+        strategy: FilterStrategy,
+        optimize_alpha: bool,
+    ) -> (Vec<u8>, FilterStrategy) {
         let mut filtered = Vec::with_capacity(self.data.len());
         let bpp = self.bytes_per_channel() * self.channels_per_pixel();
         // If alpha optimization is enabled, determine how many bytes of alpha there are per pixel
@@ -399,31 +374,39 @@ impl PngImage {
         let mut prev_line = Vec::new();
         let mut prev_pass: Option<u8> = None;
         let mut f_buf = Vec::new();
-        for line in self.scan_lines(false) {
+        // For heuristic strategies, keep track of the actual filter used for each line
+        let mut filters_used = Vec::new();
+        for (i, line) in self.scan_lines(false).enumerate() {
             if prev_pass != line.pass || line.data.len() != prev_line.len() {
                 prev_line = vec![0; line.data.len()];
             }
             // Alpha optimisation may alter the line data, so we need a mutable copy of it
             let mut line_data = line.data.to_vec();
 
-            if filter <= RowFilter::Paeth {
+            if let FilterStrategy::Basic(mut filter) = strategy {
                 // Standard filters
-                let filter = if prev_pass == line.pass || filter <= RowFilter::Sub {
-                    filter
-                } else {
-                    RowFilter::None
-                };
+                if prev_pass != line.pass && filter > RowFilter::Sub {
+                    filter = RowFilter::None;
+                }
+                filter.filter_line(bpp, &mut line_data, &prev_line, &mut f_buf, alpha_bytes);
+                filtered.extend_from_slice(&f_buf);
+                prev_line = line_data;
+            } else if let FilterStrategy::Predefined(lines) = &strategy {
+                // Predefined filter for each line
+                let filter = lines.get(i).unwrap_or(&RowFilter::None);
                 filter.filter_line(bpp, &mut line_data, &prev_line, &mut f_buf, alpha_bytes);
                 filtered.extend_from_slice(&f_buf);
                 prev_line = line_data;
             } else {
                 // Heuristic filter selection strategies
 
+                let mut best_filter = RowFilter::None;
                 if line_data.iter().all(|&x| x == 0) {
                     // Assume None if the line is all zeros
-                    filtered.push(RowFilter::None as u8);
+                    filtered.push(best_filter as u8);
                     filtered.extend_from_slice(&line_data);
                     prev_line = line_data;
+                    filters_used.push(best_filter);
                     continue;
                 }
 
@@ -431,12 +414,12 @@ impl PngImage {
                 let mut best_line_raw = Vec::new();
                 // Avoid vertical filtering on first line of each interlacing pass
                 let try_filters = if prev_pass == line.pass {
-                    RowFilter::STANDARD.iter()
+                    RowFilter::ALL.iter()
                 } else {
                     RowFilter::SINGLE_LINE.iter()
                 };
-                match filter {
-                    RowFilter::MinSum => {
+                match strategy {
+                    FilterStrategy::MinSum => {
                         // MSAD algorithm mentioned in libpng reference docs
                         // http://www.libpng.org/pub/png/book/chapter09.html
                         let mut best_size = usize::MAX;
@@ -450,10 +433,11 @@ impl PngImage {
                                 best_size = size;
                                 std::mem::swap(&mut best_line, &mut f_buf);
                                 best_line_raw.clone_from(&line_data);
+                                best_filter = *f;
                             }
                         }
                     }
-                    RowFilter::Entropy => {
+                    FilterStrategy::Entropy => {
                         // Shannon entropy algorithm, from LodePNG
                         // https://github.com/lvandeve/lodepng
                         let mut best_size = i32::MIN;
@@ -473,10 +457,11 @@ impl PngImage {
                                 best_size = size;
                                 std::mem::swap(&mut best_line, &mut f_buf);
                                 best_line_raw.clone_from(&line_data);
+                                best_filter = *f;
                             }
                         }
                     }
-                    RowFilter::Bigrams => {
+                    FilterStrategy::Bigrams => {
                         // Count distinct bigrams, from pngwolf
                         // https://bjoern.hoehrmann.de/pngwolf/
                         let mut best_size = usize::MAX;
@@ -492,10 +477,11 @@ impl PngImage {
                                 best_size = size;
                                 std::mem::swap(&mut best_line, &mut f_buf);
                                 best_line_raw.clone_from(&line_data);
+                                best_filter = *f;
                             }
                         }
                     }
-                    RowFilter::BigEnt => {
+                    FilterStrategy::BigEnt => {
                         // Bigram entropy, combined from Entropy and Bigrams filters
                         let mut best_size = i32::MIN;
                         // FxHasher is the fastest rust hasher currently available for this purpose
@@ -512,31 +498,33 @@ impl PngImage {
                                 best_size = size;
                                 std::mem::swap(&mut best_line, &mut f_buf);
                                 best_line_raw.clone_from(&line_data);
+                                best_filter = *f;
                             }
                         }
                     }
-                    RowFilter::Brute => {
+                    FilterStrategy::Brute { num_lines, level } => {
                         // Brute force by compressing each filter attempt
                         // Similar to that of LodePNG but includes some previous lines for context
                         let mut best_size = usize::MAX;
                         let line_start = filtered.len();
                         filtered.resize(filtered.len() + line.data.len() + 1, 0);
                         let mut compressor =
-                            Compressor::new(CompressionLvl::new(BRUTE_LEVEL).unwrap());
-                        let limit = filtered.len().min((line.data.len() + 1) * BRUTE_LINES);
-                        let capacity = compressor.zlib_compress_bound(limit);
+                            Compressor::new(CompressionLvl::new(level.into()).unwrap());
+                        let limit = filtered.len().min((line.data.len() + 1) * num_lines);
+                        let capacity = compressor.deflate_compress_bound(limit);
                         let mut dest = vec![0; capacity];
 
                         for f in try_filters {
                             f.filter_line(bpp, &mut line_data, &prev_line, &mut f_buf, alpha_bytes);
                             filtered[line_start..].copy_from_slice(&f_buf);
                             let size = compressor
-                                .zlib_compress(&filtered[filtered.len() - limit..], &mut dest)
+                                .deflate_compress(&filtered[filtered.len() - limit..], &mut dest)
                                 .unwrap_or(usize::MAX);
                             if size < best_size {
                                 best_size = size;
                                 std::mem::swap(&mut best_line, &mut f_buf);
                                 best_line_raw.clone_from(&line_data);
+                                best_filter = *f;
                             }
                         }
                         filtered.resize(line_start, 0);
@@ -545,11 +533,17 @@ impl PngImage {
                 }
                 filtered.extend_from_slice(&best_line);
                 prev_line = best_line_raw;
+                filters_used.push(best_filter);
             }
 
             prev_pass = line.pass;
         }
-        filtered
+
+        if filters_used.is_empty() {
+            (filtered, strategy)
+        } else {
+            (filtered, FilterStrategy::Predefined(filters_used))
+        }
     }
 }
 
