@@ -1,6 +1,6 @@
 use std::sync::Arc;
 
-use crate::{evaluate::Evaluator, png::PngImage, ColorType, Deadline, Deflaters, Options};
+use crate::{ColorType, Deadline, Deflater, Options, evaluate::Evaluator, png::PngImage};
 
 pub mod alpha;
 use crate::alpha::*;
@@ -21,8 +21,8 @@ pub(crate) fn perform_reductions(
 
     // At low compression levels, skip some transformations which are less likely to be effective
     // This currently affects optimization presets 0-2
-    let cheap = match opts.deflate {
-        Deflaters::Libdeflater { compression } => compression < 12 && opts.fast_evaluation,
+    let cheap = match opts.deflater {
+        Deflater::Libdeflater { compression } => compression < 12 && opts.fast_evaluation,
         _ => false,
     };
 
@@ -106,7 +106,9 @@ pub(crate) fn perform_reductions(
     // Attempt to convert from indexed to channels
     // This may give a better result due to dropping the PLTE chunk
     if !cheap && opts.color_type_reduction && !deadline.passed() {
-        if let Some(reduced) = indexed_to_channels(&png, opts.grayscale_reduction) {
+        if let Some(reduced) =
+            indexed_to_channels(&png, opts.grayscale_reduction, opts.optimize_alpha)
+        {
             // This result should not be passed on to subsequent reductions
             eval.try_image(Arc::new(reduced));
             evaluation_added = true;
@@ -181,7 +183,7 @@ pub(crate) fn perform_reductions(
         if (!cheap || reduced.is_none()) && !deadline.passed() {
             if let Some(indexed) = indexed.and_then(|png| reduced_bit_depth_8_or_less(&png)) {
                 // Only evaluate this if it's different from the first result (which must be grayscale if it exists)
-                if reduced.as_ref().map_or(true, |r| r.data != indexed.data) {
+                if reduced.as_ref().is_none_or(|r| r.data != indexed.data) {
                     eval.try_image(Arc::new(indexed));
                     evaluation_added = true;
                 }
