@@ -4,8 +4,7 @@ use rgb::RGBA8;
 use crate::{
     colors::{BitDepth, ColorType},
     headers::IhdrData,
-    png::{scan_lines::ScanLine, PngImage},
-    Interlacing,
+    png::{PngImage, scan_lines::ScanLine},
 };
 
 /// Attempt to reduce the number of colors in the palette, returning the reduced image if successful
@@ -87,7 +86,7 @@ pub fn sorted_palette(png: &PngImage) -> Option<PngImage> {
     let mut enumerated: Vec<_> = palette.iter().enumerate().collect();
     // Put the most popular edge color first, which can help slightly if the filter bytes are 0
     let keep_first = most_popular_edge_color(palette.len(), png);
-    let first = enumerated.remove(keep_first);
+    let first = keep_first.map(|f| enumerated.remove(f));
 
     // Sort the palette
     enumerated.sort_by(|a, b| {
@@ -104,7 +103,9 @@ pub fn sorted_palette(png: &PngImage) -> Option<PngImage> {
         };
         color_val(a.1).cmp(&color_val(b.1))
     });
-    enumerated.insert(0, first);
+    if let Some(first) = first {
+        enumerated.insert(0, first);
+    }
 
     // Extract the new palette and determine if anything changed
     let (remapping, palette): (Vec<_>, Vec<RGBA8>) = enumerated.into_iter().unzip();
@@ -132,7 +133,7 @@ pub fn sorted_palette(png: &PngImage) -> Option<PngImage> {
 #[must_use]
 pub fn sorted_palette_mzeng(png: &PngImage) -> Option<PngImage> {
     // Interlacing not currently supported
-    if png.ihdr.bit_depth != BitDepth::Eight || png.ihdr.interlaced != Interlacing::None {
+    if png.ihdr.bit_depth != BitDepth::Eight || png.ihdr.interlaced {
         return None;
     }
     let palette = match &png.ihdr.color_type {
@@ -154,7 +155,7 @@ pub fn sorted_palette_mzeng(png: &PngImage) -> Option<PngImage> {
 #[must_use]
 pub fn sorted_palette_battiato(png: &PngImage) -> Option<PngImage> {
     // Interlacing not currently supported
-    if png.ihdr.bit_depth != BitDepth::Eight || png.ihdr.interlaced != Interlacing::None {
+    if png.ihdr.bit_depth != BitDepth::Eight || png.ihdr.interlaced {
         return None;
     }
     let palette = match &png.ihdr.color_type {
@@ -204,27 +205,31 @@ fn apply_palette_reorder(png: &PngImage, remapping: &[usize]) -> Option<PngImage
 }
 
 // Find the most popular color on the image edges (the pixels neighboring the filter bytes)
-fn most_popular_edge_color(num_colors: usize, png: &PngImage) -> usize {
-    let mut counts = [0u32; 256];
+fn most_popular_edge_color(num_colors: usize, png: &PngImage) -> Option<usize> {
+    let mut counts = [0_u32; 256];
     for line in png.scan_lines(false) {
         if let &[first, .., last] = line.data {
             counts[first as usize] += 1;
             counts[last as usize] += 1;
         }
     }
-    counts
+    let max = counts
         .iter()
-        .copied()
         .take(num_colors)
         .enumerate()
         .max_by_key(|&(_, v)| v)
-        .unwrap_or_default()
-        .0
+        .unwrap();
+    // Ensure there's a clear winner - return None if multiple colors are tied
+    let max_equal = counts.iter().filter(|&v| v == max.1).count();
+    if max_equal > 1 {
+        return None;
+    }
+    Some(max.0)
 }
 
 // Find the most popular color in the image, along with its count
 fn most_popular_color(num_colors: usize, png: &PngImage) -> (usize, u32) {
-    let mut counts = [0u32; 256];
+    let mut counts = [0_u32; 256];
     for &val in &png.data {
         counts[val as usize] += 1;
     }
@@ -256,7 +261,7 @@ fn apply_most_popular_color(png: &PngImage, remapping: &mut [usize]) {
 
 // Calculate co-occurences matrix
 fn co_occurrence_matrix(num_colors: usize, png: &PngImage) -> Vec<Vec<u32>> {
-    let mut matrix = vec![vec![0u32; num_colors]; num_colors];
+    let mut matrix = vec![vec![0_u32; num_colors]; num_colors];
     let mut prev: Option<ScanLine> = None;
     let mut prev_val = None;
     for line in png.scan_lines(false) {
@@ -278,7 +283,7 @@ fn co_occurrence_matrix(num_colors: usize, png: &PngImage) -> Vec<Vec<u32>> {
                 matrix[val][prev_val] += 1;
             }
         }
-        prev = Some(line)
+        prev = Some(line);
     }
     matrix
 }
