@@ -12,20 +12,16 @@ const GRAYSCALE_ALPHA: u8 = 4;
 const RGBA: u8 = 6;
 
 fn get_opts(input: &Path) -> (OutFile, oxipng::Options) {
-    let mut options = oxipng::Options {
+    let options = oxipng::Options {
         force: true,
         ..Default::default()
     };
-    let mut filter = IndexSet::new();
-    filter.insert(RowFilter::None);
-    options.filter = filter;
-
     (OutFile::from_path(input.with_extension("out.png")), options)
 }
 
 fn test_it_converts(
     input: &str,
-    filter: RowFilter,
+    filter: FilterStrategy,
     color_type_in: u8,
     bit_depth_in: BitDepth,
     color_type_out: u8,
@@ -35,15 +31,14 @@ fn test_it_converts(
 
     let (output, mut opts) = get_opts(&input);
     let png = PngData::new(&input, &opts).unwrap();
-    opts.filter = IndexSet::new();
-    opts.filter.insert(filter);
+    opts.filters = indexset! {filter};
     assert_eq!(png.raw.ihdr.color_type.png_header_code(), color_type_in);
     assert_eq!(png.raw.ihdr.bit_depth, bit_depth_in);
 
     match oxipng::optimize(&InFile::Path(input), &output, &opts) {
         Ok(_) => (),
         Err(x) => panic!("{}", x),
-    };
+    }
     let output = output.path().unwrap();
     assert!(output.exists());
 
@@ -68,7 +63,7 @@ fn test_it_converts(
 fn filter_0_for_rgba_16() {
     test_it_converts(
         "tests/files/filter_0_for_rgba_16.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         RGBA,
         BitDepth::Sixteen,
         RGBA,
@@ -80,7 +75,7 @@ fn filter_0_for_rgba_16() {
 fn filter_1_for_rgba_16() {
     test_it_converts(
         "tests/files/filter_1_for_rgba_16.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         RGBA,
         BitDepth::Sixteen,
         RGBA,
@@ -92,7 +87,7 @@ fn filter_1_for_rgba_16() {
 fn filter_2_for_rgba_16() {
     test_it_converts(
         "tests/files/filter_2_for_rgba_16.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         RGBA,
         BitDepth::Sixteen,
         RGBA,
@@ -104,7 +99,7 @@ fn filter_2_for_rgba_16() {
 fn filter_3_for_rgba_16() {
     test_it_converts(
         "tests/files/filter_3_for_rgba_16.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         RGBA,
         BitDepth::Sixteen,
         RGBA,
@@ -116,7 +111,7 @@ fn filter_3_for_rgba_16() {
 fn filter_4_for_rgba_16() {
     test_it_converts(
         "tests/files/filter_4_for_rgba_16.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         RGBA,
         BitDepth::Sixteen,
         RGBA,
@@ -128,7 +123,7 @@ fn filter_4_for_rgba_16() {
 fn filter_5_for_rgba_16() {
     test_it_converts(
         "tests/files/filter_5_for_rgba_16.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         RGBA,
         BitDepth::Sixteen,
         RGBA,
@@ -140,7 +135,7 @@ fn filter_5_for_rgba_16() {
 fn filter_0_for_rgba_8() {
     test_it_converts(
         "tests/files/filter_0_for_rgba_8.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         RGBA,
         BitDepth::Eight,
         RGBA,
@@ -152,7 +147,7 @@ fn filter_0_for_rgba_8() {
 fn filter_1_for_rgba_8() {
     test_it_converts(
         "tests/files/filter_1_for_rgba_8.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         RGBA,
         BitDepth::Eight,
         RGBA,
@@ -164,7 +159,7 @@ fn filter_1_for_rgba_8() {
 fn filter_2_for_rgba_8() {
     test_it_converts(
         "tests/files/filter_2_for_rgba_8.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         RGBA,
         BitDepth::Eight,
         RGBA,
@@ -176,7 +171,7 @@ fn filter_2_for_rgba_8() {
 fn filter_3_for_rgba_8() {
     test_it_converts(
         "tests/files/filter_3_for_rgba_8.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         RGBA,
         BitDepth::Eight,
         RGBA,
@@ -188,7 +183,7 @@ fn filter_3_for_rgba_8() {
 fn filter_4_for_rgba_8() {
     test_it_converts(
         "tests/files/filter_4_for_rgba_8.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         RGBA,
         BitDepth::Eight,
         RGBA,
@@ -200,7 +195,7 @@ fn filter_4_for_rgba_8() {
 fn filter_5_for_rgba_8() {
     test_it_converts(
         "tests/files/filter_5_for_rgba_8.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         RGBA,
         BitDepth::Eight,
         RGBA,
@@ -212,7 +207,7 @@ fn filter_5_for_rgba_8() {
 fn filter_0_for_rgb_16() {
     test_it_converts(
         "tests/files/filter_0_for_rgb_16.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         RGB,
         BitDepth::Sixteen,
         RGB,
@@ -224,7 +219,7 @@ fn filter_0_for_rgb_16() {
 fn filter_1_for_rgb_16() {
     test_it_converts(
         "tests/files/filter_1_for_rgb_16.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         RGB,
         BitDepth::Sixteen,
         RGB,
@@ -236,7 +231,7 @@ fn filter_1_for_rgb_16() {
 fn filter_2_for_rgb_16() {
     test_it_converts(
         "tests/files/filter_2_for_rgb_16.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         RGB,
         BitDepth::Sixteen,
         RGB,
@@ -248,7 +243,7 @@ fn filter_2_for_rgb_16() {
 fn filter_3_for_rgb_16() {
     test_it_converts(
         "tests/files/filter_3_for_rgb_16.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         RGB,
         BitDepth::Sixteen,
         RGB,
@@ -260,7 +255,7 @@ fn filter_3_for_rgb_16() {
 fn filter_4_for_rgb_16() {
     test_it_converts(
         "tests/files/filter_4_for_rgb_16.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         RGB,
         BitDepth::Sixteen,
         RGB,
@@ -272,7 +267,7 @@ fn filter_4_for_rgb_16() {
 fn filter_5_for_rgb_16() {
     test_it_converts(
         "tests/files/filter_5_for_rgb_16.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         RGB,
         BitDepth::Sixteen,
         RGB,
@@ -284,7 +279,7 @@ fn filter_5_for_rgb_16() {
 fn filter_0_for_rgb_8() {
     test_it_converts(
         "tests/files/filter_0_for_rgb_8.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         RGB,
         BitDepth::Eight,
         RGB,
@@ -296,7 +291,7 @@ fn filter_0_for_rgb_8() {
 fn filter_1_for_rgb_8() {
     test_it_converts(
         "tests/files/filter_1_for_rgb_8.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         RGB,
         BitDepth::Eight,
         RGB,
@@ -308,7 +303,7 @@ fn filter_1_for_rgb_8() {
 fn filter_2_for_rgb_8() {
     test_it_converts(
         "tests/files/filter_2_for_rgb_8.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         RGB,
         BitDepth::Eight,
         RGB,
@@ -320,7 +315,7 @@ fn filter_2_for_rgb_8() {
 fn filter_3_for_rgb_8() {
     test_it_converts(
         "tests/files/filter_3_for_rgb_8.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         RGB,
         BitDepth::Eight,
         RGB,
@@ -332,7 +327,7 @@ fn filter_3_for_rgb_8() {
 fn filter_4_for_rgb_8() {
     test_it_converts(
         "tests/files/filter_4_for_rgb_8.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         RGB,
         BitDepth::Eight,
         RGB,
@@ -344,7 +339,7 @@ fn filter_4_for_rgb_8() {
 fn filter_5_for_rgb_8() {
     test_it_converts(
         "tests/files/filter_5_for_rgb_8.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         RGB,
         BitDepth::Eight,
         RGB,
@@ -356,7 +351,7 @@ fn filter_5_for_rgb_8() {
 fn filter_0_for_grayscale_alpha_16() {
     test_it_converts(
         "tests/files/filter_0_for_grayscale_alpha_16.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         GRAYSCALE_ALPHA,
         BitDepth::Sixteen,
         GRAYSCALE_ALPHA,
@@ -368,7 +363,7 @@ fn filter_0_for_grayscale_alpha_16() {
 fn filter_1_for_grayscale_alpha_16() {
     test_it_converts(
         "tests/files/filter_1_for_grayscale_alpha_16.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         GRAYSCALE_ALPHA,
         BitDepth::Sixteen,
         GRAYSCALE_ALPHA,
@@ -380,7 +375,7 @@ fn filter_1_for_grayscale_alpha_16() {
 fn filter_2_for_grayscale_alpha_16() {
     test_it_converts(
         "tests/files/filter_2_for_grayscale_alpha_16.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         GRAYSCALE_ALPHA,
         BitDepth::Sixteen,
         GRAYSCALE_ALPHA,
@@ -392,7 +387,7 @@ fn filter_2_for_grayscale_alpha_16() {
 fn filter_3_for_grayscale_alpha_16() {
     test_it_converts(
         "tests/files/filter_3_for_grayscale_alpha_16.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         GRAYSCALE_ALPHA,
         BitDepth::Sixteen,
         GRAYSCALE_ALPHA,
@@ -404,7 +399,7 @@ fn filter_3_for_grayscale_alpha_16() {
 fn filter_4_for_grayscale_alpha_16() {
     test_it_converts(
         "tests/files/filter_4_for_grayscale_alpha_16.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         GRAYSCALE_ALPHA,
         BitDepth::Sixteen,
         GRAYSCALE_ALPHA,
@@ -416,7 +411,7 @@ fn filter_4_for_grayscale_alpha_16() {
 fn filter_5_for_grayscale_alpha_16() {
     test_it_converts(
         "tests/files/filter_5_for_grayscale_alpha_16.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         GRAYSCALE_ALPHA,
         BitDepth::Sixteen,
         GRAYSCALE_ALPHA,
@@ -428,7 +423,7 @@ fn filter_5_for_grayscale_alpha_16() {
 fn filter_0_for_grayscale_alpha_8() {
     test_it_converts(
         "tests/files/filter_0_for_grayscale_alpha_8.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         GRAYSCALE_ALPHA,
         BitDepth::Eight,
         GRAYSCALE_ALPHA,
@@ -440,7 +435,7 @@ fn filter_0_for_grayscale_alpha_8() {
 fn filter_1_for_grayscale_alpha_8() {
     test_it_converts(
         "tests/files/filter_1_for_grayscale_alpha_8.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         GRAYSCALE_ALPHA,
         BitDepth::Eight,
         GRAYSCALE_ALPHA,
@@ -452,7 +447,7 @@ fn filter_1_for_grayscale_alpha_8() {
 fn filter_2_for_grayscale_alpha_8() {
     test_it_converts(
         "tests/files/filter_2_for_grayscale_alpha_8.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         GRAYSCALE_ALPHA,
         BitDepth::Eight,
         GRAYSCALE_ALPHA,
@@ -464,7 +459,7 @@ fn filter_2_for_grayscale_alpha_8() {
 fn filter_3_for_grayscale_alpha_8() {
     test_it_converts(
         "tests/files/filter_3_for_grayscale_alpha_8.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         GRAYSCALE_ALPHA,
         BitDepth::Eight,
         GRAYSCALE_ALPHA,
@@ -476,7 +471,7 @@ fn filter_3_for_grayscale_alpha_8() {
 fn filter_4_for_grayscale_alpha_8() {
     test_it_converts(
         "tests/files/filter_4_for_grayscale_alpha_8.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         GRAYSCALE_ALPHA,
         BitDepth::Eight,
         GRAYSCALE_ALPHA,
@@ -488,7 +483,7 @@ fn filter_4_for_grayscale_alpha_8() {
 fn filter_5_for_grayscale_alpha_8() {
     test_it_converts(
         "tests/files/filter_5_for_grayscale_alpha_8.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         GRAYSCALE_ALPHA,
         BitDepth::Eight,
         GRAYSCALE_ALPHA,
@@ -500,7 +495,7 @@ fn filter_5_for_grayscale_alpha_8() {
 fn filter_0_for_grayscale_16() {
     test_it_converts(
         "tests/files/filter_0_for_grayscale_16.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         GRAYSCALE,
         BitDepth::Sixteen,
         GRAYSCALE,
@@ -512,7 +507,7 @@ fn filter_0_for_grayscale_16() {
 fn filter_1_for_grayscale_16() {
     test_it_converts(
         "tests/files/filter_1_for_grayscale_16.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         GRAYSCALE,
         BitDepth::Sixteen,
         GRAYSCALE,
@@ -524,7 +519,7 @@ fn filter_1_for_grayscale_16() {
 fn filter_2_for_grayscale_16() {
     test_it_converts(
         "tests/files/filter_2_for_grayscale_16.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         GRAYSCALE,
         BitDepth::Sixteen,
         GRAYSCALE,
@@ -536,7 +531,7 @@ fn filter_2_for_grayscale_16() {
 fn filter_3_for_grayscale_16() {
     test_it_converts(
         "tests/files/filter_3_for_grayscale_16.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         GRAYSCALE,
         BitDepth::Sixteen,
         GRAYSCALE,
@@ -548,7 +543,7 @@ fn filter_3_for_grayscale_16() {
 fn filter_4_for_grayscale_16() {
     test_it_converts(
         "tests/files/filter_4_for_grayscale_16.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         GRAYSCALE,
         BitDepth::Sixteen,
         GRAYSCALE,
@@ -560,7 +555,7 @@ fn filter_4_for_grayscale_16() {
 fn filter_5_for_grayscale_16() {
     test_it_converts(
         "tests/files/filter_5_for_grayscale_16.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         GRAYSCALE,
         BitDepth::Sixteen,
         GRAYSCALE,
@@ -572,7 +567,7 @@ fn filter_5_for_grayscale_16() {
 fn filter_0_for_grayscale_8() {
     test_it_converts(
         "tests/files/filter_0_for_grayscale_8.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         GRAYSCALE,
         BitDepth::Eight,
         GRAYSCALE,
@@ -584,7 +579,7 @@ fn filter_0_for_grayscale_8() {
 fn filter_1_for_grayscale_8() {
     test_it_converts(
         "tests/files/filter_1_for_grayscale_8.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         GRAYSCALE,
         BitDepth::Eight,
         GRAYSCALE,
@@ -596,7 +591,7 @@ fn filter_1_for_grayscale_8() {
 fn filter_2_for_grayscale_8() {
     test_it_converts(
         "tests/files/filter_2_for_grayscale_8.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         GRAYSCALE,
         BitDepth::Eight,
         GRAYSCALE,
@@ -608,7 +603,7 @@ fn filter_2_for_grayscale_8() {
 fn filter_3_for_grayscale_8() {
     test_it_converts(
         "tests/files/filter_3_for_grayscale_8.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         GRAYSCALE,
         BitDepth::Eight,
         GRAYSCALE,
@@ -620,7 +615,7 @@ fn filter_3_for_grayscale_8() {
 fn filter_4_for_grayscale_8() {
     test_it_converts(
         "tests/files/filter_4_for_grayscale_8.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         GRAYSCALE,
         BitDepth::Eight,
         GRAYSCALE,
@@ -632,7 +627,7 @@ fn filter_4_for_grayscale_8() {
 fn filter_5_for_grayscale_8() {
     test_it_converts(
         "tests/files/filter_5_for_grayscale_8.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         GRAYSCALE,
         BitDepth::Eight,
         GRAYSCALE,
@@ -644,7 +639,7 @@ fn filter_5_for_grayscale_8() {
 fn filter_0_for_palette_4() {
     test_it_converts(
         "tests/files/filter_0_for_palette_4.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         INDEXED,
         BitDepth::Four,
         INDEXED,
@@ -656,7 +651,7 @@ fn filter_0_for_palette_4() {
 fn filter_1_for_palette_4() {
     test_it_converts(
         "tests/files/filter_1_for_palette_4.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         INDEXED,
         BitDepth::Four,
         INDEXED,
@@ -668,7 +663,7 @@ fn filter_1_for_palette_4() {
 fn filter_2_for_palette_4() {
     test_it_converts(
         "tests/files/filter_2_for_palette_4.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         INDEXED,
         BitDepth::Four,
         INDEXED,
@@ -680,7 +675,7 @@ fn filter_2_for_palette_4() {
 fn filter_3_for_palette_4() {
     test_it_converts(
         "tests/files/filter_3_for_palette_4.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         INDEXED,
         BitDepth::Four,
         INDEXED,
@@ -692,7 +687,7 @@ fn filter_3_for_palette_4() {
 fn filter_4_for_palette_4() {
     test_it_converts(
         "tests/files/filter_4_for_palette_4.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         INDEXED,
         BitDepth::Four,
         INDEXED,
@@ -704,7 +699,7 @@ fn filter_4_for_palette_4() {
 fn filter_5_for_palette_4() {
     test_it_converts(
         "tests/files/filter_5_for_palette_4.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         INDEXED,
         BitDepth::Four,
         INDEXED,
@@ -716,7 +711,7 @@ fn filter_5_for_palette_4() {
 fn filter_0_for_palette_2() {
     test_it_converts(
         "tests/files/filter_0_for_palette_2.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         INDEXED,
         BitDepth::Two,
         INDEXED,
@@ -728,7 +723,7 @@ fn filter_0_for_palette_2() {
 fn filter_1_for_palette_2() {
     test_it_converts(
         "tests/files/filter_1_for_palette_2.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         INDEXED,
         BitDepth::Two,
         INDEXED,
@@ -740,7 +735,7 @@ fn filter_1_for_palette_2() {
 fn filter_2_for_palette_2() {
     test_it_converts(
         "tests/files/filter_2_for_palette_2.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         INDEXED,
         BitDepth::Two,
         INDEXED,
@@ -752,7 +747,7 @@ fn filter_2_for_palette_2() {
 fn filter_3_for_palette_2() {
     test_it_converts(
         "tests/files/filter_3_for_palette_2.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         INDEXED,
         BitDepth::Two,
         INDEXED,
@@ -764,7 +759,7 @@ fn filter_3_for_palette_2() {
 fn filter_4_for_palette_2() {
     test_it_converts(
         "tests/files/filter_4_for_palette_2.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         INDEXED,
         BitDepth::Two,
         INDEXED,
@@ -776,7 +771,7 @@ fn filter_4_for_palette_2() {
 fn filter_5_for_palette_2() {
     test_it_converts(
         "tests/files/filter_5_for_palette_2.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         INDEXED,
         BitDepth::Two,
         INDEXED,
@@ -788,7 +783,7 @@ fn filter_5_for_palette_2() {
 fn filter_0_for_palette_1() {
     test_it_converts(
         "tests/files/filter_0_for_palette_1.png",
-        RowFilter::None,
+        FilterStrategy::NONE,
         INDEXED,
         BitDepth::One,
         INDEXED,
@@ -800,7 +795,7 @@ fn filter_0_for_palette_1() {
 fn filter_1_for_palette_1() {
     test_it_converts(
         "tests/files/filter_1_for_palette_1.png",
-        RowFilter::Sub,
+        FilterStrategy::SUB,
         INDEXED,
         BitDepth::One,
         INDEXED,
@@ -812,7 +807,7 @@ fn filter_1_for_palette_1() {
 fn filter_2_for_palette_1() {
     test_it_converts(
         "tests/files/filter_2_for_palette_1.png",
-        RowFilter::Up,
+        FilterStrategy::UP,
         INDEXED,
         BitDepth::One,
         INDEXED,
@@ -824,7 +819,7 @@ fn filter_2_for_palette_1() {
 fn filter_3_for_palette_1() {
     test_it_converts(
         "tests/files/filter_3_for_palette_1.png",
-        RowFilter::Average,
+        FilterStrategy::AVERAGE,
         INDEXED,
         BitDepth::One,
         INDEXED,
@@ -836,7 +831,7 @@ fn filter_3_for_palette_1() {
 fn filter_4_for_palette_1() {
     test_it_converts(
         "tests/files/filter_4_for_palette_1.png",
-        RowFilter::Paeth,
+        FilterStrategy::PAETH,
         INDEXED,
         BitDepth::One,
         INDEXED,
@@ -848,7 +843,7 @@ fn filter_4_for_palette_1() {
 fn filter_5_for_palette_1() {
     test_it_converts(
         "tests/files/filter_5_for_palette_1.png",
-        RowFilter::MinSum,
+        FilterStrategy::MinSum,
         INDEXED,
         BitDepth::One,
         INDEXED,
